## Server-Side Rendering

Server-Side Rendering(SSR) has two advantages:

* Be friendly to SEO
* Faster First Screen Display Time

Server-side rendering (SSR) also has drawbacks, such as increasing server-side overhead. Developers can weigh whether to use SSR or pre-render directly with [omi-snap](https://github.com/Tencent/omi/blob/master/tutorial/omi-snap.cn.md). Pre-rendering does not require additional server-side overhead and generates skeleton screen directly at the time of construction, so there is no dynamic data content. SSR can return HTML generated by dynamic data, and also the serialization data can be returned with HTML.

## Usage

```bash
$ npm i omi-cli -g      # install cli
$ omi init-s my-app     # init project
$ cd my-app           
$ npm start             # develop
$ npm run build         # release
```

## Principle

Define component:

```jsx
import { WeElement, define } from 'omio'

define('my-element', class extends WeElement {
  install() {
    this.data = { liked: false }
  }

  static css = 'button{ color: red; }'

  render() {
    if (this.data.liked) {
      return 'You liked this.'
    }

    return <button onClick={() => {
      this.data.liked = true
      this.update()
    }} >Like</button>
  }
})
```

Note that omio is used here, SSR can only use omio, not omi, because OMI is web components, node can not render web components.

> Both OMI and omio projects must use omio for SSR

Start a node server:

```jsx
var express = require('express')
var app = express()
var Omi = require('omio')
require('./my-element')

app.get('/', function (req, res) {
  const obj = Omi.renderToString(<my-element />)
  res.end(`<!DOCTYPE html>
<html lang="en">
  <head>
    <title>Omi SSR</title>
    ${toString(obj.css)}
  </head>
  <body>${obj.html}</body>
</html>`)
})

function toString(css){
  return (
     css.map(style => {
          return `<style id="${style.id}">${style.css}</style>`
      }
    ))
  )
}

app.listen(3000)
```

Rendering HTML structure:

```html
<!DOCTYPE html>
<html lang="en">
  <head>
    <title>Omi SSR</title>
    <style type="text/css" id="_ss0">button[_ss0]{color:red;}</style>
  </head>
  <body><button _ss0>Like</button></body>
</html>
```

Preview:

![](https://github.com/Tencent/omi/raw/master/assets/hello-ssr.png)


